﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using C1.Silverlight.Chart;

namespace DataPointCustomization
{
  public partial class MainPage : UserControl
  {
    public MainPage()
    {
      InitializeComponent();

      DefaultAppearance();
      CustomDataPointColor();
      CustomDataPointSize();
      CustomDataPointSymbol();
      CustomDataPointLabel();
    }

    /// <summary>
    /// Builds chart with default appearance.
    /// </summary>
    void DefaultAppearance()
    {
      chart0.Data.Children.Add(CreateDataSeries(50));
    }

    /// <summary>
    /// Builds chart with custom color of data points.
    /// </summary>
    void CustomDataPointColor()
    {
      var ds = CreateDataSeries(50);
      ds.PlotElementLoaded += (sender, args) =>
        {
          var pe = (PlotElement)sender;
          var dp = pe.DataPoint;
          if (!(pe is Lines)) // skip connecting lines
          {
            // fill the symbol with color that is calculated from the data value
            pe.Fill = GetBrush(Colors.Blue, Colors.White, Colors.Red, dp.Value);
          }
        };
      chart1.Data.Children.Add(ds);
    }

    /// <summary>
    /// Builds chart with custom size of data points.
    /// </summary>
    void CustomDataPointSize()
    {
      var ds = CreateDataSeries(50);
      ds.PlotElementLoaded += (s, e) =>
      {
        var pe = (PlotElement)s;
        var dp = pe.DataPoint;
        if (!(pe is Lines)) // skip connecting lines
        {
          // apply simple scale transformation depending on data point value
          double scale = 1 + 0.4 * dp.Value;
          pe.RenderTransform = new ScaleTransform() { ScaleX = scale, ScaleY = scale };
          pe.RenderTransformOrigin = new Point(0.5, 0.5);
        }
      };
      chart2.Data.Children.Add(ds);
    }
    
    /// <summary>
    /// Builds chart with arrows at the data points.
    /// </summary>
    void CustomDataPointSymbol()
    {
      var ds = CreateDataSeries(30);
      ds.ConnectionStrokeThickness = 1;
      
      ds.PlotElementLoaded += (sender, args) =>
      {
        var pe = (PlotElement)sender; 
        var dp = pe.DataPoint as XYDataSeries.XYDataPoint;
        if (!(pe is Lines))  // skip connecting lines
        {
          pe.Opacity = 0.5;
          var pnl = pe.Parent as Panel;
          if (pnl != null)
          {
            var x = (double)dp.X;
            // find angle of tangent
            var angle = Math.Atan( Math.Cos(x)/ GetAspectRatio(chart3));  
            // create arrow
            var arrow = CreateArrow( angle, 20);
            arrow.Fill = arrow.Stroke = new SolidColorBrush(Colors.Red);
            // set position(base) of arrow 
            Canvas.SetLeft(arrow, pe.Center.X);
            Canvas.SetTop(arrow, pe.Center.Y);
            // put arrow on the plot
            pnl.Children.Add(arrow);
          }
        }
      };
      chart3.Data.Children.Add(ds);
    }

    /// <summary>
    /// Builds chart with annotations.
    /// </summary>
    void CustomDataPointLabel()
    {
      var ds = CreateDataSeries(50);

      ds.PlotElementLoaded += (sender, args) =>
      {
        var pe = (PlotElement)sender;
        var dp = pe.DataPoint;
        if (!(pe is Lines)  // skip connecting lines
        && (dp.PointIndex % 5) == 1 ) // each 5-th point
        {
          var pnl = pe.Parent as Panel;
          if (pnl != null)
          {
            // create and add label
            var tb = new TextBlock() { Text = dp.Value.ToString("0.00") };
            var bdr = new Border()
            {
              Child = tb, Padding=new Thickness(2,0,2,0),
              Background = new SolidColorBrush(Colors.White) { Opacity=0.7},
              BorderBrush = new SolidColorBrush(Colors.Gray),
              BorderThickness = new Thickness(2, 2, 1, 1),
              CornerRadius = new CornerRadius(0, 5, 5, 5)
            }; 
            // set label position
            Canvas.SetLeft(bdr, pe.Center.X);
            Canvas.SetTop(bdr, pe.Center.Y);
            pnl.Children.Add(bdr);
          }
        }
      };
      chart4.Data.Children.Add(ds);
    }

    #region Auxiliary routines

    /// <summary>
    /// Create brush mixing three colors in the proportion specified by value(-1,+1).
    /// </summary>
    /// <param name="clr1"></param>
    /// <param name="clr2"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    SolidColorBrush GetBrush(Color clr1, Color clr2, Color clr3, double value)
    {
      if (value > 0)
      {
        clr1 = clr2; clr2 = clr3;
      }

      Color clr = Color.FromArgb((byte)(clr1.A + value * (clr2.A - clr1.A)),
        (byte)(clr1.R + value * (clr2.R - clr1.R)),
        (byte)(clr1.G + value * (clr2.G - clr1.G)),
        (byte)(clr1.B + value * (clr2.B - clr1.B)));
      return new SolidColorBrush(clr);
    }

    /// <summary>
    /// Gets the aspect ration between data and screen coordinates
    /// </summary>
    /// <param name="chart"></param>
    /// <returns></returns>
    double GetAspectRatio(C1Chart chart)
    {
      Point pt1 = chart.View.PointToData(new Point());
      Point pt2 = chart.View.PointToData(new Point(1,1));
      return (pt2.Y - pt1.Y) / (pt2.X - pt1.X);
    }

    /// <summary>
    /// Creates sample data series.
    /// </summary>
    /// <param name="npts"></param>
    /// <returns></returns>
    XYDataSeries CreateDataSeries(int npts)
    {
      double[] x = new double[npts];
      double[] y = new double[npts];
      for (int i = 0; i < npts; i++)
      {
        x[i] = 0.5*i; y[i] = Math.Sin(x[i]);
      }

      return new XYDataSeries() { XValuesSource = x, ValuesSource = y };
    }

    /// <summary>
    /// Creates arrow with the specified angle and length.
    /// </summary>
    /// <param name="angle"></param>
    /// <param name="len"></param>
    /// <returns></returns>
    Path CreateArrow(double angle, double len)
    {
      const double arr_tip_angle = 0.9*Math.PI;
      double arr_tip_len = 0.5 * len;
      Point pt = new Point(len * Math.Cos(angle), len * Math.Sin(angle));
      Point pt1 = new Point(pt.X + arr_tip_len * Math.Cos(angle-arr_tip_angle), 
        pt.Y + arr_tip_len * Math.Sin(angle - arr_tip_angle));
      Point pt2 = new Point(pt.X + arr_tip_len * Math.Cos(angle + arr_tip_angle),
        pt.Y + arr_tip_len * Math.Sin(angle + arr_tip_angle)); 

      var path = new Path();
      GeometryGroup gg = new GeometryGroup();
      gg.Children.Add(new LineGeometry() { EndPoint = pt });

      var pg = new PathGeometry();
      var pf = new PathFigure();
      pf.StartPoint = pt;
      pf.Segments.Add(new LineSegment() { Point = pt1 });
      pf.Segments.Add(new LineSegment() { Point = pt2 });
      pg.Figures.Add(pf);
      gg.Children.Add(pg);

      path.Data = gg;
      return path;
    }
    #endregion
  }
}
